// ----------------------------------------------------------------------- //
// _gfx.cpp		initializing stuff for d3d
//
// last edited:	2001-05-11 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// includes

#include "stdafx.h"
#include "d3d8.h"

#include "_gfx.h"
#include "_shared.h"

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// variables

LPDIRECT3D8				lpd3d;		// the direct 3d object
LPDIRECT3DDEVICE8		lpd3dd;		// our direct 3d device

float					_aspect;	// aspect ratio
int						_gfx_initdone = 0;

// ----------------------------------------------------------------------- //




// initializes direct3d

int		_gfx_init					// returns a nonzero value if successful
	( 
		int nWidth,					// desired width
		int nHeight,				// and height of screen
		int bFullscreen				// set to 1 for fullscreen
	)
{
	// create d3d object
	if (FAILED( lpd3d = Direct3DCreate8( D3D_SDK_VERSION ) ))
	{
		_log_printf("failed to create direct 3d object");
		return 0;
	}

	// find displaymode
	D3DDISPLAYMODE			d3ddm;

	if (FAILED( lpd3d->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ))
	{
		_log_printf("failed to retrieve current display mode");
		return 0;
	}

	// set parameters
	D3DPRESENT_PARAMETERS	d3dpp;

	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.Windowed				 = !bFullscreen;			// fullscreen?
	d3dpp.BackBufferFormat		 = D3DFMT_X8R8G8B8;			// 32 bit
	d3dpp.BackBufferWidth		 = nWidth;
	d3dpp.BackBufferHeight		 = nHeight;
	d3dpp.BackBufferCount		 = 1;
	d3dpp.SwapEffect             = D3DSWAPEFFECT_DISCARD;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

	// 32bit hw -> 32bit sw -> 24bit hw -> 24bit sw -> 16bit hw -> 16bit sw -> exit

	_log_printf("trying %u x %u x 32", nWidth, nHeight);

	// create device - first try hardware vertex processing
	if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
										D3DCREATE_HARDWARE_VERTEXPROCESSING,
										&d3dpp, &lpd3dd ) ) )
										// if that fails software processing
	if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
										D3DCREATE_SOFTWARE_VERTEXPROCESSING,
										&d3dpp, &lpd3dd ) ) )
	{
		_log_printf("failed ... trying %u x %u x 24", nWidth, nHeight);
		// if failed try this:
		d3dpp.BackBufferFormat		 = D3DFMT_R8G8B8;	// 24 bit
		// create device - first try hardware vertex processing

		if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
											D3DCREATE_HARDWARE_VERTEXPROCESSING,
											&d3dpp, &lpd3dd ) ) )
											// if that fails software processing
		if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
											D3DCREATE_SOFTWARE_VERTEXPROCESSING,
											&d3dpp, &lpd3dd ) ) )
		{
			_log_printf("failed ... trying %u x %u x 16", nWidth, nHeight);
			// if failed try this:
			d3dpp.BackBufferFormat		 = D3DFMT_R5G6B5;	// 16 bit
			if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
												D3DCREATE_HARDWARE_VERTEXPROCESSING,
												&d3dpp, &lpd3dd ) ) )
												// if that fails software processing
			if( FAILED( lpd3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
												D3DCREATE_SOFTWARE_VERTEXPROCESSING,
												&d3dpp, &lpd3dd ) ) )
			{
				_log_printf("failed ...");
				return 0;
			}
		}
	}

	_log_printf("done ...");

	// calculate aspect ratio
	_aspect = 1.0f*nWidth / (nHeight?nHeight*1.0f:1.0f);

	if (!bFullscreen)
		if (_aspect<1) _aspect = 1;

	_gfx_initdone = 1;

	_log_printf("_gfx_init() is done");

	return 1;
} // _gfx_init



// called when window is resized

int		_gfx_resize					// returns always 1
	( 
		int nWidth,					// new width
		int nHeight,				// and height of window
		int bFullscreen				// useless
	)
{
	// only if everything is already initialized
	if (!_gfx_initdone)
		return 0;

	// recalculate aspect ratio
	_aspect = 1.0f*nWidth / (nHeight?nHeight*1.0f:1.0f);

	if (!bFullscreen)
		if (_aspect<1) _aspect = 1;

	return 1;
} // _gfx_resize



// releases d3d objects
int		_gfx_deinit()
{
	_log_printf("_gfx_deinit()");

	SafeRelease(lpd3dd);
	SafeRelease(lpd3d);
	return 1;
} // _gfx_deinit

// ----------------------------------------------------------------------- //
