// ----------------------------------------------------------------------- //
// _images.cpp	image loading stuff
//
// last edited:	2001-05-11 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// notes
/*

 - _image_mirror_v mirrors the image linewise because large images
   can cause a buffer that is to big to malloc

*/



// ----------------------------------------------------------------------- //
// includes

#include "stdio.h"
#include "stdafx.h"
#include "stdlib.h"

#include "_shared.h"
#include "_images.h"

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// image functions



// frees an image loaded with _load_image_buf
void	_free_image
	( 
		imageinfo_t *info			// pointer to the image info to free
	)
{
	if (!info) return;

	// first free its buffer
	if (info->buf) 
		free(info->buf);

	// then free itself
	if (info->freeable)
		free(info);
} // _free_image



// mirrors an image vertically (eg. for tgas and jpgs that are bottomup)

void	_image_mirror_v
	( 
		imageinfo_t *info			// pointer to an image descriptor
	)
{
	char	*b = NULL;

	// allocate memory for one line
	b = (char*)malloc( 4 * info->width );
	if (!b) return;

	// for every line
	for (int i=0; i<info->height/2; i++)
	{
		int h2 = info->height - i - 1;

		// save line i to buffer
		memcpy( b, info->buf+info->width * 4 * i, 4*info->width );

		// move line h2 to line i
		memcpy( info->buf+info->width * 4 * i, info->buf + 4*info->width * h2, 4*info->width );

		// move saved line to line h2
		memcpy( info->buf + 4*info->width * h2, b, 4 * info->width );
	}

	free( b );
} // _image_mirror_v


// loads an image from a buffer

imageinfo_t	*_load_image			// returns an image descriptor
	( 
		file_t *file,				// file to read from
		int type					// type of image
	)
{
	switch(type)
	{
		case IMAGE_TGA:	return _load_tga( file );
	}
	return NULL;
}

// ----------------------------------------------------------------------- //
