// ----------------------------------------------------------------------- //
// _r_font.cpp	text drawing stuff
//
// last edited:	2001-05-11 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// notes
/*

 - _r_draw_string_var uses an array of floats to determine the distance 
   between to chars. each entry must hold the width for its character
   corresponding character relative to height so 1 means the character
   is height wide. 0 is replaced with 1 :)

 */
// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// includes

#include "_gfx.h"
#include "_render.h"

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// variables

flexVert_t	vCharVerts[4];

extern	LPDIRECT3DVERTEXBUFFER8		vertBuf;
extern	DWORD						dwFvf;

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// functions

// this draws a character

void	_r_draw_char
	( 
		float x,					// left and
		float y,					// bottom coordinates
		float size,					// height of character
		char c,						// character to draw
		int tex						// texture to use
	)
{
	_r_draw_pic( tex, x,y, x+size, y+size, (c%16)/16.0f, (c/16+1)/16.0f, (c%16+1)/16.0f, (c/16)/16.0f );
} // _r_draw_char



// draws a string with same width as size

void	_r_draw_string
	(
		float x,					// left and
		float y,					// bottom coordinates
		float size,					// height of character
		char *c,					// string
		int tex						// texture to use
	)
{
	float	cx = x-size;

	while (*c) _r_draw_char(cx+=size,y, size, *(c++), tex);
} // _r_draw_string



// draws a string with chars as big as size but always dist units apart

void	_r_draw_string_dist
	( 
		float x,					// left and
		float y,					// bottom coordinates
		float size,					// height
		float dist,					// distance between chars
		char *c,					// string	
		int tex						// texture
	)
{
	float	cx = x-dist;

	while (*c) _r_draw_char(cx+=dist,y, size, *(c++), tex);
} // _r_draw_string_dist




// draws a string with a variable distance between chars

void	_r_draw_string_var
	( 
		float x,					// coordinates
		float y, 
		float size,					// height
		float *dist,				// pointer to an array of floats
		char *c,					// string
		int tex						// texture
	)
{
	float	cx = x;
	float	d;

	while (*c)
	{
		d = dist[*c]*size;
		if (!d) d = size;
		_r_draw_char(cx,y, size, *(c++), tex);
		cx+=d;
	}
} // _r_draw_string_var

// ----------------------------------------------------------------------- //
