// ----------------------------------------------------------------------- //
// _r_textures.cpp	texture loading and stuff
//
// last edited:	2001-05-16 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// notes
/*

 - when loading or creating a texture a return value of -1 means an error
   occured.

 - if a texture was not found, MAX_TEXTURES-1 is returned

 - texture names are without extensions!

 */
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// includes

#include "stdio.h"
#include "stdafx.h"

#include "_gfx.h"
#include "_shared.h"
#include "_render.h"
#include "_images.h"

// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// variables

texture	_r_textures[MAX_TEXTURES];	// the buffer that holds all textures
int		actualtexture = -1;			// contains the actual set texture

// ----------------------------------------------------------------------- //




// ----------------------------------------------------------------------- //
// functions


// finds the number of a texture

int		_r_texture_find				// returns the number of the requested texture
	( 
		char *szName				// name of texture to find
	)
{
	for (int i=1; i < MAX_TEXTURES; i++)
	{
		if (!strcmpi(szName, _r_textures[i].szTextureName))
			return i;
	}

	return MAX_TEXTURES-1;
} // _r_texture_find




// finds the first free texture slot

int		_r_texture_findfree			// returns the number of a free texture
	()
{
	// exclude MAX_TEXTURES-1 from search because this is the "not found" texture

	for (int i=MAX_TEXTURES-2; i; i--)
	{
		if (_r_textures[i].pTexture) continue;

		return i;
	}

	return -1;
} // _r_texture_findfree




// create a texture in a given slot

int		_r_texture_create			// returns num if successful .. 
	( 
		int width,					// requested width
		int height,					// and height
		char *szName,				// and name of the texture
		int num						// slot to create the texture in
	)
{
	if( !FAILED( D3DXCreateTexture( 
			lpd3dd, width, height, 0, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, 
			&_r_textures[num].pTexture )))
	{
		_snprintf(_r_textures[num].szTextureName, 64, "%s", szName);
		_snprintf(_r_textures[num].szFileName, MAX_PATH, "");
		_r_textures[num].Info.Width = width;
		_r_textures[num].Info.Height = height;
		return num;
	}

	return -1;
} // _r_texture_create




// create a texture in any slot

int		_r_texture_create			// returns the number of the slot it was saved in
	( 
		int width,					// requested width
		int height,					// and height
		char *szName				// and name of the texture
	)
{
	int num;

	num = _r_texture_findfree();
	if (num == -1)
		return num;

	return _r_texture_create( width, height, szName, num );
} // _r_texture_create




// load a texture to a given slot

int		_r_texture_load				// return the number of the texture
	( 
		char *szFile,				// filename of texture
		char *szName,				// texture name
		int num,					// slot to load texture to
		int type					// type of image (default: IMAGE_TGA)
	)
{
	imageinfo_t		*ii;			// the image descriptoer
	file_t			*file;			// the file descriptor
	
	D3DLOCKED_RECT	lrect;			// for locking the texture

	_convert_path(szFile);

	// load file
	file = _file_load( szFile ); 
	if (!file) return -1;

	// load image
	ii = _load_image( file, type );
	if (!ii) return -1;

	// create texture
	if ( _r_texture_create( ii->width, ii->height, szName, num ) == -1 ) 
		return -1;

	// copy data into texture
	_r_textures[num].pTexture->LockRect( 0, &lrect, NULL, 0 );

	// pixelwise
	for (int y=0; y<ii->height; y++)
			for (int x=0; x<ii->width; x++)
			{
				char	*wp = (char*)lrect.pBits+lrect.Pitch*y+x*4;
				char	*rp = ii->buf+y*4*ii->width+x*4;
				memcpy( (void*)wp, (void*)rp, 4 );
			}

	_r_textures[num].pTexture->UnlockRect( 0 );

	// filter texture (create mipmaps)
	D3DXFilterTexture( _r_textures[num].pTexture, NULL, 0, D3DX_DEFAULT );
	
	return num;
} // _r_texture_load




// load a texture to a given slot

int	_r_texture_load					// returns the slot of the texture
	(
		char *szName,				// name of texture to load
		int num						// slot we would like it to be in
	)
{
	char	szFilename[MAX_PATH];	// filename

	_log_printf("loading texture \"%s\" as #%u", szName, num);

	// if slot is busy.. free it
	if ( _r_textures[num].pTexture )
		_r_textures[num].pTexture->Release();

	_r_textures[num].pTexture = NULL;

	// TODO: other image formats

	// try loading as a targa
	sprintf(szFilename, "%s.tga", szName);
	if ( _r_texture_load(szFilename, szName, num, IMAGE_TGA )+1)
		return num;

	_log_printf("failed ... ");
	return -1;
} // _r_texture_load




// load a texture to any slot

int	_r_texture_load					// returns the number of the used slot
	(
		char *szName				// name of the texture
	)
{
	int		t;						// temp int

	// if texture already exists return its number
	t = _r_texture_find( szName );
	if (t != MAX_TEXTURES-1) return t;

	// otherwise find a free slot
	t = _r_texture_findfree();

	// and load the texture into it
	if (t != -1 )
		t = _r_texture_load( szName, t );

	return t;
} // _r_texture_load





// frees all textures

void	_r_texture_freeall()
{
	for (int i=0; i< MAX_TEXTURES; i++)
		SafeRelease( _r_textures[i].pTexture );
} // _r_texture_freeall




// sets the current texture

void	_r_texture_set
	( 
		int num,					// number of texture to use
		int stage					// for what texture state (default: 0)
	)
{
	// dont do anything if texture is already set
	if (actualtexture == num) return;

	// if there is a texture use it.. otherwise use "not found" texture
	if (!_r_textures[num].pTexture)
		lpd3dd->SetTexture( stage, _r_textures[MAX_TEXTURES-1].pTexture );
	else
		lpd3dd->SetTexture( stage, _r_textures[num].pTexture );
} // _r_texture_set




// creates mipmaps

void	_r_texture_filter 
	(
		int	num						// number of texture
	)
{
	D3DXFilterTexture( _r_textures[num].pTexture, NULL, 0, D3DX_DEFAULT );
}


// ----------------------------------------------------------------------- //
