// ----------------------------------------------------------------------- //
// _main.cpp	main file
//
// last edited:	2001-05-12 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// includes

#include "stdio.h"
#include "dsound.h"

#include "_shared.h"
#include "_sound.h"

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// variables

sound_t			_s_sounds[MAX_SOUNDS];

LPDIRECTSOUND8	lpds; 

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// functions


// initializes the sound engine

int		_sound_init					// returns nonzero if successful
	( 
		HWND hWnd					// window handle
	)
{
	HRESULT hr;
	
	if (FAILED(hr = DirectSoundCreate8(NULL, &lpds, NULL)))
		return 0;

	hr = lpds->Initialize(NULL);

	lpds->SetCooperativeLevel( hWnd, DSSCL_PRIORITY );

	for (int i=0; i<MAX_SOUNDS; i++)
	{
		_s_sounds[i].sound = 0;
	}

	return 1;
} // _sound_init




// frees a sound

void	_sound_free( int n )
{
	if (_s_sounds[n].sound)
		_s_sounds[n].sound->Release();
} // _sound_init




// deinitializes sound

void	_sound_deinit()
{
	for (int i=0; i<MAX_SOUNDS; i++)
	{
		_sound_free(i);
	}

	if (lpds)
		lpds->Release();

} // _sound_deinit




// plays a sound 

void	_sound_playsound
	(
		int n,						// number of sound to play
		long flags,					// flags for playing 
		long volume,				// volume (default: 0)
		long pan,					// pan (default: center)
		long freq					// frequency (default: original)
	)
{
	LPDIRECTSOUNDBUFFER sb = _s_sounds[n].sound;

	// if its not a looping sound we can duplicate the buffer
	if (!(flags&DSBPLAY_LOOPING))
		lpds->DuplicateSoundBuffer( sb, &sb );

	// set attributes
	sb->SetVolume(volume);
	sb->SetPan(pan);
	sb->SetFrequency(freq);

	// play!
	sb->Play(0,0,flags);
} // _sound_playsound




// stops a playing sound (only useful for looping)

void	_sound_stopsound(int n)
{
	LPDIRECTSOUNDBUFFER sb = _s_sounds[n].sound;

	sb->Stop();
} // _sound_stopsound




// finds a sound by its name

int		_sound_find					// returns the number of sound
	( 
		char *szName				// name of sound
	)
{
	for (int i=0; i < MAX_SOUNDS; i++)
	{
		if (!strcmpi(szName, _s_sounds[i].szSoundName))
			return i;
	}

	return -1;
} // _sound_find





// loads a sound

int		_sound_load					// returns num if successfull
	( 
		const char *szF,			// filename of sound
		char *szName,				// name of sound
		int num,					// number of slot to load to
		int flags					// loading flags
	)
{
	char			szFile[MAX_PATH];
	WAVEFORMATEX	wfm;
	file_t			*file;

	_snprintf(szFile, MAX_PATH-1, "%s", szF);
	_convert_path( szFile );

	_sound_free(num);

	file = _file_load( szFile );
	if (!file) return -1;

	// NOTE: does this really always work?

	DWORD size;
	memset(&wfm, 0, sizeof(wfm));
	
	_file_seek( file, 16, SEEK_SET );
	_file_read( (char*)&size, sizeof(DWORD), 1, file );
	_file_read( (char*)&wfm, sizeof(wfm)-sizeof(WORD), 1, file );
	_file_seek( file, sizeof(DWORD), SEEK_CUR );
	_file_read( (char*)&size, sizeof(DWORD), 1, file );

	DSBUFFERDESC desc;

	desc.dwSize = sizeof(desc);
	desc.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY;
	desc.dwBufferBytes = size;
	desc.lpwfxFormat = &wfm;
	desc.dwReserved = 0;

	lpds->CreateSoundBuffer(&desc, &_s_sounds[num].sound, NULL );

	if (!_s_sounds[num].sound)
	{
		free(file->buf);
		return -1;
	}

	char	*soundbuf;
	
	_s_sounds[num].sound->Lock( 0, size, (LPVOID*)&soundbuf, &size, NULL, NULL, DSBLOCK_ENTIREBUFFER );

	_file_read( soundbuf, size, 1, file );

	_s_sounds[num].sound->Unlock( soundbuf, size, NULL, NULL );

	_file_close(file);

	if (!_s_sounds[num].sound)
		return -1;

	return num;
} // _sound_load




// load a sound to any slot

int	_sound_load						// returns the number of used slot
	( 
		char *szName,				// name of sound
		int flags					// flags
	)
{
	int		t;

	t = _sound_find( szName );
	if (t != -1) return t;

	for (int i=MAX_SOUNDS; i; i--)
	{
		if (_s_sounds[i].sound) continue;

		if ( (t = _sound_load(szName, szName, i, flags))+1 )
			return t;
	}

	return -1;
} // _sound_load

// ----------------------------------------------------------------------- //
