// ----------------------------------------------------------------------- //
// _images.cpp	image loading stuff
//
// last edited:	2001-05-12 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// notes:
/*

 - only uncompressed rgb targa images are supported by now

*/
// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// includes

#include "stdio.h"
#include "stdafx.h"
#include "stdlib.h"

#include "_shared.h"
#include "_images.h"

// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// variables

// targa image data types
#define			IDT_RGB			2

// vars
int				_tga_imagedatatype;
unsigned char	_tga_idfieldsize;
int				_tga_bottomup;
int				_tga_interleave;
long			_tga_dataofs;
char			_tga_bpp;
imageinfo_t		*_tga_info;
int				_tga_origx, _tga_origy;

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// functions



// load an uncompressed rgb targa image

imageinfo_t	*_load_tga_rgb			// returns a pointer to an image descriptor
	(
		file_t *file				// the file descriptor of the .tga
	)
{
	char	*wp;					// temporary write pointer
	char	r,g,b,a;				// temp. vars for reg green blue and alpha

	// create buffer
	_tga_info->buflen	= _tga_info->width * _tga_info->height * 4;
	_tga_info->buf		= (char*)malloc(_tga_info->buflen);

	if (!_tga_info->buf) 
	{ 
		free(_tga_info); 
		_tga_info = NULL;
		return NULL; 
	}

	// set writepointer to beginning
	wp	= _tga_info->buf;

	// if image is bottomup, goto last line
	if ( _tga_bottomup )
		wp += _tga_info->buflen - _tga_info->width * 4;

	// seek to beginning of data
	_file_seek( file, _tga_dataofs, SEEK_SET );

	// read the pixels
	while( !_file_eof(file) && wp >= _tga_info->buf && wp < _tga_info->buf+_tga_info->buflen)
	{
		_file_getpixel( file, &r, &g, &b, &a, _tga_bpp );
		
		*wp = r; wp++;
		*wp = g; wp++;
		*wp = b; wp++;
		*wp = a; wp++;

		if ( _tga_bottomup && (wp-_tga_info->buf) % (_tga_info->width * 4) == 0 )
			wp -= _tga_info->width * 4 * 2;
	}

	return _tga_info;
} // _load_tga_rgb




// loads a targa file

imageinfo_t	*_load_tga				// returns a pointer to an image descriptor
	(
		file_t *file				// the file descriptor of the .tga
	)
{
	if (!file)
		return NULL;

	_file_getc(file);

	_tga_idfieldsize	= _file_getc(file);
	_tga_imagedatatype	= _file_getc(file);

	_tga_info			= (imageinfo_t*)malloc(sizeof(imageinfo_t));
	if (!_tga_info) return NULL;

	_tga_info->freeable = 1;
	_tga_info->buf		= NULL;

	_file_seek( file, 8, SEEK_SET );
	_tga_origx			= _file_geti(file);
	_tga_origy			= _file_geti(file);

	_tga_info->width	= _file_geti(file);
	_tga_info->height	= _file_geti(file);

	_tga_bpp			= _file_getc(file);

	char bb = _file_getc(file);
	_tga_bottomup		= !(bb & 1<<5);
	_tga_interleave		= bb & 3<<6;

	_tga_dataofs		= 18+_tga_idfieldsize;

	switch(_tga_imagedatatype)
	{
		case IDT_RGB:		return _load_tga_rgb( file );
		default:
			return NULL;
	}

	return NULL;
} // _load_tga

// ----------------------------------------------------------------------- //
