// ----------------------------------------------------------------------- //
// fire.cpp		element 2 - fire
//
// last edited:	2001-05-12 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// include 

#include "windows.h"
#include "_window.h"
#include "stdio.h"
#include "math.h"

#include "_shared.h"
#include "_render.h"
#include "_input.h"

#include "states.h"
#include "fire.h"

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// variables

int		firetex;					// texture for fire

int		firebuf[FIRE_WIDTH][FIRE_HEIGHT];			// buffers for fire
int		firebuf2[FIRE_WIDTH][FIRE_HEIGHT];			

float	nextfireupdate = 0;			// next time the fire should be moved

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// functions



// initializes all fire stuff

void init_fire()
{
	// create texture
	firetex = _r_texture_create( FIRE_WIDTH, FIRE_HEIGHT, "*fire" );

	// clear buffers
	memset(firebuf, 0, sizeof(firebuf));
	memset(firebuf2, 0, sizeof(firebuf));

} // init_fire




// updates the fire

void update_fire()
{

	nextfireupdate = gv.statetime+0.075;

	for (int x=20; x<43; x++)
		firebuf[x][63] = 255-rand()%(int)(32+31*sin(gv.statetime*5+x/8))-rand()%(int)(32+31*cos(-gv.statetime*7+x/3));

	for (x=1; x<63; x++)
		for (int y=63; y<127; y++)
		{
			firebuf2[x][y] = firebuf[x][y-1*rand()%2];
			firebuf2[x][y] += firebuf[x-1*rand()%2][y-1*rand()%2];
			firebuf2[x][y] += firebuf[x+1*rand()%2][y-1*rand()%2];
		}

	memcpy(firebuf, firebuf2, sizeof(firebuf));

	for (x=0; x<63; x++)
		for (int y=63; y<127; y++)
		{
			for (int yy=-1; yy<2; yy++)
			{
				firebuf2[x][y] += firebuf[x][y+yy];
				firebuf2[x][y] += firebuf[x-1][y+yy];
				firebuf2[x][y] += firebuf[x+1][y+yy];
			}
			firebuf2[x][y] /= 31;
			if (firebuf2[x][y]>255) firebuf2[x][y] = 255;
			if (firebuf2[x][y]<0) firebuf2[x][y] = 0;
		}

	D3DLOCKED_RECT	lr;
	char *buf;

	memcpy(firebuf, firebuf2, sizeof(firebuf));

	_r_textures[firetex].pTexture->LockRect(0, &lr, NULL, 0);

	buf = (char*)lr.pBits;

	for (x=1; x<63; x++)
		for (int y=63; y<127; y++)
		{
			int b = firebuf[x][y]*4-256-256; if (b>255) b = 255; if (b<0) b =0;
			int g = firebuf[x][y]*2-128; if (g>255) g = 255; if (g<0) g =0;
			int r = firebuf[x][y];
			float d = _vector_length(gv.viewPos)/5;

			d*=d;
			if (d<1) d=1;

			buf[x*4+y*lr.Pitch+0] = b/d;
			buf[x*4+y*lr.Pitch+1] = g/d;
			buf[x*4+y*lr.Pitch+2] = r/d;
		}

	_r_textures[firetex].pTexture->UnlockRect(0);
} // update_fire

// renders the fire

void render_fire()
{
	vec3_t p;						// fire position
	vec2_t s;						// size

	// set states
	lpd3dd->SetRenderState		( D3DRS_ALPHABLENDENABLE,	TRUE			);
	lpd3dd->SetRenderState		( D3DRS_FOGENABLE,			FALSE			);
	lpd3dd->SetRenderState		( D3DRS_ZENABLE,			D3DZB_FALSE		);
	lpd3dd->SetRenderState		( D3DRS_ZFUNC,				D3DCMP_LESSEQUAL);
	lpd3dd->SetRenderState		( D3DRS_CULLMODE,			D3DCULL_NONE	);
	lpd3dd->SetTextureStageState( 0, D3DTSS_ALPHAOP,		D3DTOP_DISABLE );
	lpd3dd->SetTextureStageState( 1, D3DTSS_COLOROP,		D3DTOP_DISABLE );

	lpd3dd->SetRenderState( D3DRS_SRCBLEND,					D3DBLEND_ONE  );
	lpd3dd->SetRenderState( D3DRS_DESTBLEND,				D3DBLEND_ONE );

	lpd3dd->SetRenderState		( D3DRS_ZWRITEENABLE,		FALSE		);

	if (gv.statetime>=nextfireupdate)
		update_fire();

	_vector_set( p, 0, -0.25, 0 );
	s[0] = 2; s[1] = -8;

	_r_draw_billboard( p, s, firetex );
} // render_fire

// ----------------------------------------------------------------------- //
