#include "windows.h"
#include "_window.h"
#include "stdio.h"
#include "math.h"
#include "dsound.h"

#include "_shared.h"
#include "_render.h"
#include "_sound.h"
#include "_input.h"

#include "states.h"
#include "earth.h"
#include "fire.h"
#include "water.h"
#include "wind.h"

#define	LIGHTMETHOD D3DTOP_MODULATE2X

// sounds
int		rainsound;
int		thundersound[3];
int		bgthundersound;

// textures
int		shadetex;
int		fonttex;
int		lightningtex;


// fog variables
float	fognear = 1;
float	fogfar	= 18;

// view
float	yaw = 0;
float	pitch =0;
float   viewspeed = 0;

// lightning bolt
vec3_t	lightningpos;				// position
float	lastlightning = 0;			// time of last activation
float	lightning = 0;				// amount of lightning

void state1_world_init()
{
	gv.fov = 45;

 	lpd3dd->SetRenderState( D3DRS_FOGTABLEMODE,			D3DFOG_LINEAR );
	lpd3dd->SetRenderState( D3DRS_FOGEND,				*(DWORD*)&fogfar );

	// textures
	shadetex		= _r_texture_load( "textures/shade" );
	fonttex   		= _r_texture_load( "textures/bigchars" );
	lightningtex	= _r_texture_load( "textures/bolt" );

	lpd3dd->SetRenderState( D3DRS_DITHERENABLE,			TRUE );

	// lightmap stuff -------------------------------------

	lpd3dd->SetTextureStageState( 1, D3DTSS_ADDRESSU,  D3DTADDRESS_CLAMP );
	lpd3dd->SetTextureStageState( 1, D3DTSS_ADDRESSV,  D3DTADDRESS_CLAMP );

	// filtering
	lpd3dd->SetTextureStageState( 1, D3DTSS_MINFILTER, D3DTEXF_POINT );
	lpd3dd->SetTextureStageState( 1, D3DTSS_MIPFILTER, FILTERMETHOD );
	lpd3dd->SetTextureStageState( 1, D3DTSS_MAGFILTER, FILTERMETHOD );

	// texture stuff --------------------------------------

	lpd3dd->SetTextureStageState( 0, D3DTSS_MINFILTER, FILTERMETHOD );
	lpd3dd->SetTextureStageState( 0, D3DTSS_MIPFILTER, FILTERMETHOD );
	lpd3dd->SetTextureStageState( 0, D3DTSS_MAGFILTER, FILTERMETHOD );

	init_earth();
	init_water();
	init_fire();
	init_wind();

	// initialize view
	_vector_set( gv.viewPos, -5,0,0 );
	gv.viewPos[1] = getheight(gv.viewPos[0], gv.viewPos[2])+2;
	yaw = 0; pitch = 0;

	// load sounds
	rainsound		= _sound_load("sound/rain.wav",0);
	thundersound[0] = _sound_load("sound/thunder1.wav",0);
	thundersound[1] = _sound_load("sound/thunder2.wav",0);
	thundersound[2] = _sound_load("sound/thunder3.wav",0);
	bgthundersound	= _sound_load("sound/bgthunder.wav",0);

	// loop rain
	_sound_playsound( rainsound, DSBPLAY_LOOPING, -500 );
}

void state1_world_update()
{
	float	r;
	vec3_t	v,v2;

	// update view
	yaw		+= _i_mousex/10.0f;
	pitch	-= _i_mousey/10.0f;

	// calculate view direction
	r = cos(pitch/360*PI2); // 1 for horizontal 0 for vertical view
	_vector_set( v, cos(yaw/360*PI2)*r, sin(pitch/360*PI2), sin(yaw/360*PI2)*r );

	// if right mouse button is pressed speed up... otherwise slow down
	if (_i_mouseb & 2)
		viewspeed += gv.dtime*2;
	else
		viewspeed -= gv.dtime*5;

	if (viewspeed<0) 
		viewspeed=0;
	if (viewspeed>5) 
		viewspeed=5;

	// move the view
	_vector_scale(v, gv.dtime*viewspeed, v2);
	_vector_add(v2, gv.viewPos, gv.viewPos );
	gv.viewPos[1] = getheight(gv.viewPos[0], gv.viewPos[2])+2;

	// clip the position
	if (_vector_length(gv.viewPos)>50)
	{
		_vector_normalize(gv.viewPos, gv.viewPos);
		_vector_scale(gv.viewPos, 50, gv.viewPos);
	}
	
	// calculate view target
	_vector_add( gv.viewPos, v, gv.viewTarget );
	_vector_set( gv.viewUp, 0, 1, 0 );


	// reduce lightning amount
	lightning /= 1+gv.dtime;

	// if last lightning was at least 1 sek ago and with a 1:500 probability 
	if (gv.statetime>lastlightning+1 && !(rand()%500))
	{
		// increse lightning amount
		lightning		= 10+rand()%15;
		lastlightning	= gv.statetime;

		// set the lightning position near viewer...
		do
		{
			_vector_set( lightningpos, rand()%100/100.0*50-25, 10, rand()%100/100.0*50-25 );

			// but not closer than 20
		} while ( _vector_length( lightningpos ) < 20 );
	
		_vector_add( lightningpos, gv.viewPos, lightningpos );

		// find out spatial settings for sound
		_vector_subtract( lightningpos, gv.viewPos, v2 );
		_vector_normalize( v2, v2 );

		// and play a random thunder sound
		_sound_playsound(thundersound[rand()%3], 0, 0, -4000*_vector_dot( v2, gv.viewRight ) );
	}

	// from time to time play that far away thunder sound
	if (!(rand()%100))
		_sound_playsound(bgthundersound, 0, 0, rand()%8000-4000 );

	// calculate and set new background and fog color
	char b = rand()%100/100.0f*lightning;

	gv.bg_red	= gv.bg_green = gv.bg_blue = 128;
	gv.bg_blue	+= b*2; 
	gv.bg_green += b; 
	gv.bg_red	+= b;

	lpd3dd->SetRenderState( D3DRS_FOGCOLOR,				D3DCOLOR_XRGB(gv.bg_red, gv.bg_green, gv.bg_blue) );

	// vary fog distance a bit
	fognear = 5+5*sin(gv.statetime);
	lpd3dd->SetRenderState( D3DRS_FOGSTART,				*(DWORD*)&fognear );
}

void state1_render_2d()
{
	lpd3dd->SetRenderState( D3DRS_FOGENABLE,			FALSE );
	lpd3dd->SetRenderState( D3DRS_ALPHABLENDENABLE,		TRUE );
	lpd3dd->SetRenderState( D3DRS_SRCBLEND,				D3DBLEND_SRCALPHA );
	lpd3dd->SetRenderState( D3DRS_DESTBLEND,			D3DBLEND_INVSRCALPHA );
	lpd3dd->SetTextureStageState( 0, D3DTSS_ALPHAOP,	D3DTOP_ADD );
	lpd3dd->SetTextureStageState( 1, D3DTSS_COLOROP,	D3DTOP_DISABLE );
	lpd3dd->SetRenderState( D3DRS_ZENABLE,				D3DZB_FALSE );
	
	char s[64];
	// sprintf(s, "fps: %f", gv.frame/gv.time );
	sprintf(s, "%f %f %f", gv.viewPos[0], gv.viewPos[1], gv.viewPos[2] );

	_r_draw_string(0,0,16,s,fonttex);
}

void draw_lightning() 
{
	vec2_t	s;

	lpd3dd->SetRenderState		( D3DRS_FOGENABLE,			FALSE			);

	s[0] = 4-8*rand()%2; s[1] = 32;
	if ((int)(rand()%2+lightning/2-1)>0)
		_r_draw_billboard(lightningpos, s, lightningtex);
}

void state1_render_3d()
{
	render_earth();
	render_water();
	render_wind();
	render_fire();
	draw_lightning();
	render_rain();

}